@@ -363,6 +363,12 @@ class BdCliClient(BdClientBase):
             BdCommandError: If issue not found
         """
         data = await self._run_command("show", params.issue_id)
+        # bd show returns an array, extract first element
+        if isinstance(data, list):
+            if not data:
+                raise BdCommandError(f"Issue not found: {params.issue_id}")
+            data = data[0]
+        
         if not isinstance(data, dict):
             raise BdCommandError(f"Invalid response for show {params.issue_id}")
 
@@ -421,16 +427,24 @@ class BdCliClient(BdClientBase):
             args.extend(["--assignee", params.assignee])
         if params.title:
             args.extend(["--title", params.title])
+        if params.description:
+            args.extend(["--description", params.description])
         if params.design:
             args.extend(["--design", params.design])
         if params.acceptance_criteria:
-            args.extend(["--acceptance-criteria", params.acceptance_criteria])
+            args.extend(["--acceptance", params.acceptance_criteria])
         if params.notes:
             args.extend(["--notes", params.notes])
         if params.external_ref:
             args.extend(["--external-ref", params.external_ref])
 
         data = await self._run_command(*args)
+        # bd update returns an array, extract first element
+        if isinstance(data, list):
+            if not data:
+                raise BdCommandError(f"Issue not found: {params.issue_id}")
+            data = data[0]
+        
         if not isinstance(data, dict):
             raise BdCommandError(f"Invalid response for update {params.issue_id}")
 
@@ -484,8 +498,8 @@ class BdCliClient(BdClientBase):
             self.bd_path,
             "dep",
             "add",
-            params.from_id,
-            params.to_id,
+            params.issue_id,
+            params.depends_on_id,
             "--type",
             params.dep_type,
             *self._global_flags(),
