@@ -200,10 +200,32 @@ class BdDaemonClient(BdClientBase):
 
         Raises:
             DaemonNotRunningError: If daemon is not running
+            DaemonConnectionError: If connection fails
+            DaemonError: If request fails
         """
         data = await self._send_request("ping", {})
         return json.loads(data) if isinstance(data, str) else data
 
+    async def health(self) -> Dict[str, Any]:
+        """Get daemon health status.
+
+        Returns:
+            Dict with health info including:
+            - status: "healthy" | "degraded" | "unhealthy"
+            - version: daemon version string
+            - uptime: uptime in seconds
+            - db_response_time_ms: database ping time
+            - active_connections: number of active connections
+            - memory_bytes: memory usage
+
+        Raises:
+            DaemonNotRunningError: If daemon is not running
+            DaemonConnectionError: If connection fails
+            DaemonError: If request fails
+        """
+        data = await self._send_request("health", {})
+        return json.loads(data) if isinstance(data, str) else data
+
     async def init(self, params: Optional[InitParams] = None) -> str:
         """Initialize new beads database (not typically used via daemon).
 
@@ -279,6 +301,8 @@ class BdDaemonClient(BdClientBase):
             args["assignee"] = params.assignee
         if params.title is not None:
             args["title"] = params.title
+        if params.description is not None:
+            args["description"] = params.description
 
         data = await self._send_request("update", args)
         return Issue(**(json.loads(data) if isinstance(data, str) else data))
@@ -413,8 +437,8 @@ class BdDaemonClient(BdClientBase):
             params: Dependency parameters
         """
         args = {
-            "from_id": params.from_id,
-            "to_id": params.to_id,
+            "from_id": params.issue_id,
+            "to_id": params.depends_on_id,
             "dep_type": params.dep_type or "blocks",
         }
         await self._send_request("dep_add", args)
@@ -431,7 +455,7 @@ class BdDaemonClient(BdClientBase):
         except (DaemonNotRunningError, DaemonConnectionError, DaemonError):
             return False
 
-    def close(self) -> None:
+    def cleanup(self) -> None:
         """Close daemon client connections and cleanup resources.
         
         This is called during MCP server shutdown to ensure clean termination.
