@@ -1,6 +1,10 @@
 """MCP tools for beads issue tracker."""
 
+import asyncio
 import os
+import subprocess
+from contextvars import ContextVar
+from functools import lru_cache
 from typing import Annotated, TYPE_CHECKING
 
 from .bd_client import create_bd_client, BdClientBase, BdError
@@ -25,10 +29,15 @@ from .models import (
     UpdateIssueParams,
 )
 
-# Global client instance - initialized on first use
-_client: BdClientBase | None = None
-_version_checked: bool = False
-_client_registered: bool = False
+# ContextVar for request-scoped workspace routing
+current_workspace: ContextVar[str | None] = ContextVar('workspace', default=None)
+
+# Connection pool for per-project daemon sockets
+_connection_pool: dict[str, BdClientBase] = {}
+_pool_lock = asyncio.Lock()
+
+# Version checking state (per-pool client)
+_version_checked: set[str] = set()
 
 # Default constants
 DEFAULT_ISSUE_TYPE: IssueType = "task"
@@ -50,41 +59,182 @@ def _register_client_for_cleanup(client: BdClientBase) -> None:
         pass
 
 
-async def _get_client() -> BdClientBase:
-    """Get a BdClient instance, creating it on first use.
+def _resolve_workspace_root(path: str) -> str:
+    """Resolve workspace root to git repo root if inside a git repo.
+    
+    Args:
+        path: Directory path to resolve
+        
+    Returns:
+        Git repo root if inside git repo, otherwise the original path
+    """
+    try:
+        result = subprocess.run(
+            ["git", "rev-parse", "--show-toplevel"],
+            cwd=path,
+            capture_output=True,
+            text=True,
+            check=False,
+        )
+        if result.returncode == 0:
+            return result.stdout.strip()
+    except Exception:
+        pass
+    
+    return os.path.abspath(path)
 
-    Performs version check on first initialization.
+
+@lru_cache(maxsize=128)
+def _canonicalize_path(path: str) -> str:
+    """Canonicalize workspace path to handle symlinks and git repos.
+    
+    This ensures that different paths pointing to the same project
+    (e.g., via symlinks) use the same daemon connection.
+    
+    Args:
+        path: Workspace directory path
+        
+    Returns:
+        Canonical path (handles symlinks and submodules correctly)
+    """
+    # 1. Resolve symlinks
+    real = os.path.realpath(path)
+    
+    # 2. Check for local .beads directory (submodule edge case)
+    # Submodules should use their own .beads, not the parent repo's
+    if os.path.exists(os.path.join(real, ".beads")):
+        return real
+    
+    # 3. Try to find git toplevel
+    # This ensures we connect to the right daemon for the git repo
+    return _resolve_workspace_root(real)
+
+
+async def _health_check_client(client: BdClientBase) -> bool:
+    """Check if a client is healthy and responsive.
+    
+    Args:
+        client: Client to health check
+        
+    Returns:
+        True if client is healthy, False otherwise
+    """
+    # Only health check daemon clients
+    if not hasattr(client, 'ping'):
+        return True
+    
+    try:
+        await client.ping()
+        return True
+    except Exception:
+        # Any exception means the client is stale/unhealthy
+        return False
+
+
+async def _reconnect_client(canonical: str, max_retries: int = 3) -> BdClientBase:
+    """Attempt to reconnect to daemon with exponential backoff.
+    
+    Args:
+        canonical: Canonical workspace path
+        max_retries: Maximum number of retry attempts (default: 3)
+        
+    Returns:
+        New client instance
+        
+    Raises:
+        BdError: If all reconnection attempts fail
+    """
+    use_daemon = os.environ.get("BEADS_USE_DAEMON", "1") == "1"
+    
+    for attempt in range(max_retries):
+        try:
+            client = create_bd_client(
+                prefer_daemon=use_daemon,
+                working_dir=canonical
+            )
+            
+            # Verify new client works
+            if await _health_check_client(client):
+                _register_client_for_cleanup(client)
+                return client
+                
+        except Exception:
+            if attempt < max_retries - 1:
+                # Exponential backoff: 0.1s, 0.2s, 0.4s
+                backoff = 0.1 * (2 ** attempt)
+                await asyncio.sleep(backoff)
+            continue
+    
+    raise BdError(
+        f"Failed to connect to daemon after {max_retries} attempts. "
+        "The daemon may be stopped or unresponsive."
+    )
+
+
+async def _get_client() -> BdClientBase:
+    """Get a BdClient instance for the current workspace.
+    
+    Uses connection pool to manage per-project daemon sockets.
+    Workspace is determined by current_workspace ContextVar or BEADS_WORKING_DIR env.
+
+    Performs health check before returning cached client.
+    On failure, drops from pool and attempts reconnection with exponential backoff.
+    
+    Performs version check on first connection to each workspace.
     Uses daemon client if available, falls back to CLI client.
 
     Returns:
-        Configured BdClientBase instance (config loaded automatically)
+        Configured BdClientBase instance for the current workspace
 
     Raises:
-        BdError: If bd is not installed or version is incompatible
+        BdError: If no workspace is set, or bd is not installed, or version is incompatible
     """
-    global _client, _version_checked, _client_registered
-    if _client is None:
-        # Check if daemon should be used (default: yes)
-        use_daemon = os.environ.get("BEADS_USE_DAEMON", "1") == "1"
-        workspace_root = os.environ.get("BEADS_WORKING_DIR")
-        
-        _client = create_bd_client(
-            prefer_daemon=use_daemon,
-            working_dir=workspace_root
+    # Determine workspace from ContextVar or environment
+    workspace = current_workspace.get() or os.environ.get("BEADS_WORKING_DIR")
+    if not workspace:
+        raise BdError(
+            "No workspace set. Either provide workspace_root parameter or call set_context() first."
         )
-        
-        # Register for cleanup on first creation
-        if not _client_registered:
-            _register_client_for_cleanup(_client)
-            _client_registered = True
+    
+    # Canonicalize path to handle symlinks and deduplicate connections
+    canonical = _canonicalize_path(workspace)
+    
+    # Thread-safe connection pool access
+    async with _pool_lock:
+        if canonical in _connection_pool:
+            # Health check cached client before returning
+            client = _connection_pool[canonical]
+            if not await _health_check_client(client):
+                # Stale connection - remove from pool and reconnect
+                del _connection_pool[canonical]
+                if canonical in _version_checked:
+                    _version_checked.remove(canonical)
+                
+                # Attempt reconnection with backoff
+                client = await _reconnect_client(canonical)
+                _connection_pool[canonical] = client
+        else:
+            # Create new client for this workspace
+            use_daemon = os.environ.get("BEADS_USE_DAEMON", "1") == "1"
+            
+            client = create_bd_client(
+                prefer_daemon=use_daemon,
+                working_dir=canonical
+            )
+            
+            # Register for cleanup
+            _register_client_for_cleanup(client)
+            
+            # Add to pool
+            _connection_pool[canonical] = client
+    
+    # Check version once per workspace (only for CLI client)
+    if canonical not in _version_checked:
+        if hasattr(client, '_check_version'):
+            await client._check_version()
+        _version_checked.add(canonical)
 
-    # Check version once per server lifetime (only for CLI client)
-    if not _version_checked:
-        if hasattr(_client, '_check_version'):
-            await _client._check_version()
-        _version_checked = True
-
-    return _client
+    return client
 
 
 async def beads_ready_work(
@@ -175,15 +325,32 @@ async def beads_update_issue(
     priority: Annotated[int | None, "New priority (0-4)"] = None,
     assignee: Annotated[str | None, "New assignee"] = None,
     title: Annotated[str | None, "New title"] = None,
+    description: Annotated[str | None, "Issue description"] = None,
     design: Annotated[str | None, "Design notes"] = None,
     acceptance_criteria: Annotated[str | None, "Acceptance criteria"] = None,
     notes: Annotated[str | None, "Additional notes"] = None,
     external_ref: Annotated[str | None, "External reference (e.g., gh-9, jira-ABC)"] = None,
-) -> Issue:
+) -> Issue | list[Issue]:
     """Update an existing issue.
 
     Claim work by setting status to 'in_progress'.
+    
+    Note: Setting status to 'closed' or 'open' will automatically route to
+    beads_close_issue() or beads_reopen_issue() respectively to ensure
+    proper approval workflows are followed.
     """
+    # Smart routing: intercept lifecycle status changes and route to dedicated tools
+    if status == "closed":
+        # Route to close tool to respect approval workflows
+        reason = notes if notes else "Completed"
+        return await beads_close_issue(issue_id=issue_id, reason=reason)
+    
+    if status == "open":
+        # Route to reopen tool to respect approval workflows
+        reason = notes if notes else "Reopened"
+        return await beads_reopen_issue(issue_ids=[issue_id], reason=reason)
+    
+    # Normal attribute updates proceed as usual
     client = await _get_client()
     params = UpdateIssueParams(
         issue_id=issue_id,
@@ -191,6 +358,7 @@ async def beads_update_issue(
         priority=priority,
         assignee=assignee,
         title=title,
+        description=description,
         design=design,
         acceptance_criteria=acceptance_criteria,
         notes=notes,
@@ -227,8 +395,8 @@ async def beads_reopen_issue(
 
 
 async def beads_add_dependency(
-    from_id: Annotated[str, "Issue that depends on another (e.g., bd-2)"],
-    to_id: Annotated[str, "Issue that blocks or is related to from_id (e.g., bd-1)"],
+    issue_id: Annotated[str, "Issue that has the dependency (e.g., bd-2)"],
+    depends_on_id: Annotated[str, "Issue that issue_id depends on (e.g., bd-1)"],
     dep_type: Annotated[
         DependencyType,
         "Dependency type: blocks, related, parent-child, or discovered-from",
@@ -237,22 +405,22 @@ async def beads_add_dependency(
     """Add a dependency relationship between two issues.
 
     Types:
-    - blocks: to_id must complete before from_id can start
+    - blocks: depends_on_id must complete before issue_id can start
     - related: Soft connection, doesn't block progress
     - parent-child: Epic/subtask hierarchical relationship
-    - discovered-from: Track that from_id was discovered while working on to_id
+    - discovered-from: Track that issue_id was discovered while working on depends_on_id
 
     Use 'discovered-from' when you find new work during your session.
     """
     client = await _get_client()
     params = AddDependencyParams(
-        from_id=from_id,
-        to_id=to_id,
+        issue_id=issue_id,
+        depends_on_id=depends_on_id,
         dep_type=dep_type,
     )
     try:
         await client.add_dependency(params)
-        return f"Added dependency: {from_id} depends on {to_id} ({dep_type})"
+        return f"Added dependency: {issue_id} depends on {depends_on_id} ({dep_type})"
     except BdError as e:
         return f"Error: {str(e)}"
 
